# 视频会议交接
webrtc:为了满足多方进行音视频会议，方便进行沟通，特开发此项目

## 下载项目依赖
    npm install

## 本地启动
    npm start

## 编译命令
    NODE_ENV=production npm run yun; cd yun; zip -r ../meeting.zip .

### 代码结构说明
```sh
├── node_modules             # 项目依赖模块目录
├── public                   # 文件入口
├── scripts                  # 自动化发布配置文件
├── src                      # 项目源码
│   ├── main.js              # 程序入口文件
│   ├── App.vue              # 页面入口文件
│   ├── api                  # API请求目录
│   ├── assets               # 资源目录
│   │   ├── imgs             # 图片资源
│   │   └── style            # 全局样式设置
│   ├── common               # 公用函数库
│   ├── lang                 # 国际化
│   ├── router               # 路由配置文件
│   ├── store                # 组件状态管理目录
│   │   ├── modules          # 各模块对应状态管理目录
│   │   ├── getters.js       # store的计算属性
│   │   ├── index.js         # 加载各store模块
│   │   └── types.js         # 定义事件类型
│   ├── views                # 组件文件
│   │   ├── component        # 页面下子组件目录(包括公共组件)
│   │   └── page             # 页面文件目录
├── .editorconfig            # 编辑器的环境配置
├── .env.xx                  # 各个环境配置url
├── .gitignore               # 提交代码忽略的文件目录配置
├── .npmrc                   # npm镜像url
├── babel.config.js          # babel配置
├── ci.yml                   # 编译构建场景
├── jsconfig.json            # 添加引入变量自动识别 @ 符号 到 src 目录
├── package-lock.json        # 项目安装包依赖相关信息
├── package.json             # 项目依赖配置文件
├── prettier.config.js       # 配置prettier
├── README.md                # 说明文档
└── vue.config.js            # 配置文件
```

## 页面划分及主要功能
-   1. 会前页(Home)
    主要功能点
    1)会议信息展示：展示当前会议的起止时间、会议名称以及当前会议的状态(状态不同，需要展示也不同) 具体状态说明可查看下方的 ### 会议信息展示状态说明
    2)初始化sdk：为了展示视频画面，测试麦克风、扬声器效果等
    3)设置摄像头、麦克风的入会状态（打开 | 关闭）
        通过gVideoMuted，gMuted两个字段控制摄像头、麦克风的展示状态
        gVideoMuted(摄像头)  true:打开摄像头 false:关闭摄像头
        gMuted(麦克风)       true:静音 false:非静音
    4)设备调试：具体说明在第三点 设备调试(TestDialog)

-   2. 入会页(Join)
    1)上方展示会议信息及会议开始时间
    2)下方展示工具栏（会邀链接，布局切换，摄像头、麦克风、共享的开关，闭锁|解锁房间，设备调试，退出会议）
    3)左侧展示当前会议所有人的状态（姓名、摄像头、麦克风、共享、头像颜色）
      列表的摄像头、麦克风状态说明
        video  true打开摄像头 false:关闭摄像头
        audio  true:静音     false:非静音
    4)主窗口展示视频窗口（共有两个视频布局模式：画廊模式、语音激励模式）目前默认画廊
        画廊模式：    所有人的窗口统一大小
        语音激励模式： 声音最大的在大窗口，其余人在右侧小窗口
        两个模式下都是最多展示四个人的窗口，超出隐藏，
        两种模式可进行切换(如果会议中其他人打开了共享，那么会自动改成语音激励模式，并且不可切换，自己开共享可切换)
    5)设备调试：具体说明在第三点 设备调试(TestDialog)

-   3. 设备调试(TestDialog)
    主要功能点：
    1)摄像头：调用测试的api展示视频画面
    2)音频测试：通过播放一段音乐来测试当前扬声器设备
    3)设备切换：手动选择设备进行切换（摄像头、麦克风、扬声器）
    4)展示当前语言：（zh:中文  en:英文  ja:日文）

### 具体功能点说明
-   1. 语音激励效果
        是每两秒轮询的获取远端音频列表，每次查找一下列表的中声音(audioLevel)最大值，并且大于 0.05，那么给这个人的窗口添加蓝色边框和正在说话的波纹，放到第一位，如果是语音激励模式，则把这个人放到大屏（有人开共享，只需要加蓝框，不改变窗口位置）；如果这个人的dom事隐藏的，那么需要让这个人的dom可见。同时，判断这个人的volume是否=0，如果是，则重新拉流
-   2. 每十秒轮询的获取远端音频列表
        每次查找一下列表的中声音(audioLevel)最大值，并且大于 0.05，判断volume是否=0，如果是，重新拉流(因为2秒和10秒的定时器会有间隔特别短的时候，如果声音最大的用户的volume=0,那么会触发两次重新拉流，个人认为应该在10秒这去掉声音最大并且volume=0的重新拉流，但沟通无果，暂保留)；除声音最大的之外，如果同一个人的两次获取的volume都为0的话，重新拉流并且上报（上报的具体情况说明已在上报处加注释）
-   3. 展示当前会议开始时长
        是在sdk初始化成功后调用的计时方法this.timeInterval(),从00:00开始计时
-   4. 重新推流
        1)现在推流是自动模式(autopublish设为true),如果推流失败，mediastate函数会返回false,然后进行对应的上报和重新推流
        2)在success函数里判断是否触发了localvideopublished_ok，进行相应的上报和重新推流
-   5. 闭锁|解锁房间 
        会议信息的接口的cookie会携带is_owner=字段，如果等于1,代表当前用户为会议主持人，可以闭锁|解锁房间，并且调用后端接口。room_status=2是闭锁  room_status=3是解锁(只有主持人可以闭锁|解锁房间，is_owner !== 1:非主持人)
-   6. 开关摄像头、麦克风、共享等
        都是通过方法传值操作的，具体调用在下方sdk说明里，调用完毕后需要通过消息通道的方法发送给其他人：调用BRTC_SetUserAttribute(data)方法，其他人就会通过BRTC_Start里的onattribute函数接收到传的数据来进行相应的状态展示
-   7. 左侧列表点击某人
        相当于手动给这个人加语音激励效果
-   8. 蒙层说明
        一个不显示视频流的dom,主要目的是展示大屏人的状态
            1）模式：只在语音激励模式下才会存在蒙层，所以要在切换模式时判断，如果切换到语音激励就创建蒙层，切换到画廊就删除蒙层
            2）人数：只有房间总人数超过两人才会显示蒙层窗口，所以涉及到远端加入/离开，需要增/删蒙层（触发进入后房间人数大于两人，创建蒙层；触发离开后房间人数小于等于两人，删除蒙层）(只是语音激励模式)
            3）有远端进入时如果共享为打开状态，需要自动切换到语音激励模式，开共享的一方放到大屏，此时语音激励效果不生效(包括菜单点击) (唯一生效的只有给说话声音最大的人加蓝框，位置不变)
-   9. 有人进入会议后执行的逻辑
        会得到这个人的信息(id,姓名,状态等)，根据id创建dom,然后调用BRTC_SubscribeStreaming(domId, id),传domId和得到的id,如果此时房间已存在人数超过4人,则需要停掉这个人的视频画面,即调用BRTC_MuteRemoteVideo(true, id);反之,则调用BRTC_MuteRemoteVideo(false, id);

-   10. 有人退出会议
        只需要删除退出人的dom即可


### 重新拉流
    是指调用sdk的BRTC_StopSubscribeStreaming和BRTC_SubscribeStreaming,方法的具体使用在下方的 ### sdk方法说明
-   1. 每个窗口右上角的刷新按钮会重新拉流
-   2. 在每十秒获取的音频列表里会有判断是否需要重新拉流
-   3. 在每两秒获取的音频列表里会有判断是否需要重新拉流

### 上报状态说明（是指触发下面的操作都需要调用后端接口）
    0、加入
    1、退出
    2、sdk报错导致异常退出（重新刷新页面）
    3、volume为0 & 麦克风为打开状态
    4、音频推流失败
    5、视频推流失败
    6、共享失败
    7、sdk执行推流回调上报
    8、5秒内推流没有回调上报
    9、15秒后重新推流没有回调上报
    10、人员列表里某人不在音频列表里，重新拉流

### sdk方法说明
-  1. BRTC_Start(); // 初始化sdk（在后端接口返回状态和其他判断通过后调用）
-  2. BRTC_Stop(); // 停用sdk（在beforeDestroy停用）
-  3. BRTC_SwitchScreen();  // 共享和摄像头切换  会在onlocalstream函数里告知当
        前状态是共享还是摄像头
-  4. BRTC_MuteMicphone(); // 是否静音 true是
-  5. BRTC_MuteCamera(); // 摄像头状态 true关闭   false打开
-  6. BRTC_SetUserAttribute(); // 向远端发送数据（在初始进入成功、麦克风,摄像头,
        共享,闭锁|解锁房间状态改变时发送数据）

    // 静默视频
-  7. BRTC_MuteRemoteVideo(true,  id); // 停止拉此人的视频画面
-  8. BRTC_MuteRemoteVideo(false,  id,  20000); // 拉此人的视频画面
        // BRTC_MuteRemoteVideo(flag, id); // 拉取或停用某个视频的视频流  true:停用  false:拉取  为false时设置第三个时间参数(单位)  指超过多少毫秒重新启用，打开视频流
        // 因为支持多方入会，目前右侧列表最多展示四方，超出的窗口隐藏,需要把隐藏的
        视频dom的BRTC_MuteRemoteVideo设为true
        // 假如第5个人(隐藏)声音最大，需要把他放到前方，把他的BRTC_MuteRemoteVideo设为false；原来的第4个人会被隐藏，把他的BRTC_MuteRemoteVideo设为true.

    // 拉取或停用音视频流
-  9. BRTC_SubscribeStreaming(dom, id); // 拉取音视频流
    // 一般在remotevideocoming(有远端进入)时触发，注意：在调用之前需要创建dom,传值两个分别为（domId, remotevideocoming返回的用户id）
-  10. BRTC_StopSubscribeStreaming(id); // 停用音视频流
    // 现在项目中由于某些用户发生异常情况，导致听不到声音或者看不到视频等，需要重新拉流，
    也就是再次调用BRTC_SubscribeStreaming和BRTC_StopSubscribeStreaming。
    // 注意：需要先调用BRTC_StopSubscribeStreaming停用视频流，给一定间隔后再调用BRTC_SubscribeStreaming，现在项目中有多处都调用了重新拉流的逻辑，若优化的话，可以封装一个方法

-  11. BRTC_StopPublish(); // 停止推流
-  12. BRTC_StartPublish(); // 开始推流

-  13. BRTC_GetVideoDevices(); // 获取摄像头列表
-  14. BRTC_ReplaceVideo(id) 切换摄像头（应用场景是具有多个可视频设备的切换，传对
        应的deviceId即可）
-  15. BRTC_GetAudioInputDevices(); // 获取麦克风列表
-  16. BRTC_ReplaceAudio(id); 切换麦克风（应用场景是具有多个可音频输入设备的切换，
        传对应的deviceId即可）
-  17. BRTC_GetAudioOutputDevices(); // 获取扬声器列表
-  18. BRTC_SetAudioOutputDevice(id); 切换扬声器（应用场景是具有多个可音频输出
        设备的切换，传对应的deviceId即可）
-  19. BRTC_SetDevicesChangeListener(); // 监听设备插入、拔出事件(包括蓝牙)
-  20. BRTC_GetRemoteAudioLevels(); // 获取远端音频列表

    // 会前页sdk调用(测试api)
-  21. BRTC_DeviceTest_Start()   初始化start
-  22. BRTC_DeviceTest_Stop()     停掉sdk
-  23. BRTC_DeviceTest_MuteCamera()  是否静默视频 true:关摄像头  false:开摄像头
-  24. BRTC_DeviceTest_MuteMicphone()  是否静默音频  true:禁用麦克风  false:启动麦克风
-  25. BRTC_DeviceTest_SetAudioOutputDevice(deviceId) // 切换扬声器,传设备id
-  26. BRTC_DeviceTest_ReplaceVideo(deviceId) // 切换摄像头,传设备id
-  27. BRTC_DeviceTest_ReplaceAudio(deviceId) // 切换麦克风,传设备id

### 分支说明
- dev          测试环境
- yun          百度y云

### 环境说明
- env.development：
    本地调试
    访问地址：localhost:端口

- env.yun：百度云
    icode（yun分支）
    在yun分支单独部署，提供了百度云的入会后页面，会前页面是在百度云，约定好传几个固定的
    参数过来：
    {
        r:会议的唯一标识,
        name:参会人名字,
        camera:摄像头状态,
        microphone:麦克风状态,
        lang:当前项目的语言（zh|不传:中文，en:英文，ja:日文）
    }

### 会议信息展示状态说明
    当前会议状态共6个：
    未开始                  0 （显示‘未开始’文本，禁止入会）
    会议开始前1小时内         1 （可以入会）
    会议中	                2 （可以入会）
    会议锁定                3 （显示‘会议锁定，联系主持人’文本）
    会议人满	            4 （显示‘人已满，联系主持人’）
    会议结束	            5 （显示‘会议不存在’）
    会议取消	            6 （显示‘会议不存在’）

### 常见问题以及注意事项
-  1. sdk版本升级：只需要把public/index.html的外链的xx.sdk.js升级到指定版本然后部署
-  2. 如果想增加根据环境进行判断行为，可在.env.xx文件里增加字段，例：VUE_APP_ENV
-  3. scr/lang是国际化相关，所有的中英日文都在对应的文件里，目前国际化只针对百度云，
      百度云url传来什么语言，前端就展示什么（en: 英文  ja:日文  zh或者不传:中文）
-  4. 当前代码较冗余，如果重构可以继续stage分支或者重新开发
      核心思路为两个数组，即展示左侧菜单列表的数组和所有窗口列表的数组
        菜单数组：每个人的id、姓名、颜色、摄像头、麦克风、共享状态等
        窗口数组：每个人的id、姓名、颜色、摄像头、麦克风、共享状态等
        区别是菜单数组是展示当前会议所有人的数据，窗口数组展示所有窗口以及移动位置和展示各种状态等

