/**
 * @file utils
 * @author brtc
 */
/* eslint-disable */
// 深拷贝
export function deepCopy(obj, cache = []) {
    function find(list, f) {
        return list.filter(f)[0];
    }

    // just return if obj is immutable value
    if (obj === null || typeof obj !== 'object') {
        return obj;
    }

    // if obj is hit, it is in circular structure
    const hit = find(cache, c => c.original === obj);
    if (hit) {
        return hit.copy;
    }
    const copy = Array.isArray(obj) ? [] : {};
    // put the copy into cache a first
    // because we want to refer it in recursive deepCopy
    cache.push({original: obj, copy});

    Object.keys(obj).forEach(key => {
        copy[key] = deepCopy(obj[key], cache);
    });
    return copy;
}

/**
 * 获取浏览器 版本号、名称
 * @return {name: 'chrome', version: 70}
 */
export function getCurrentBrowserProfile() {
    const ua = navigator.userAgent.toLowerCase(); // 获取用户端信息
    const info = {
        ie: /msie/.test(ua) && !/opera/.test(ua), // 匹配IE浏览器
        opera: /opera/.test(ua), // 匹配Opera浏览器
        safari: /version.*safari/.test(ua), // 匹配Safari浏览器
        chrome: /chrome/.test(ua), // 匹配Chrome浏览器
        firefox: /gecko/.test(ua) && !/webkit/.test(ua) // 匹配Firefox浏览器
    };
    let target = {};
    for (const item of Object.keys(info)) {
        if (info[item]) {
            const versionStartStr = item === 'safari' ? 'version' : item;
            const reg = new RegExp(`${versionStartStr}/[0-9]+`, 'g');
            const match = ua.match(reg) || [''];
            const version = (match[0]).replace(`${versionStartStr}/`, '') - 0;
            target = {...target, name: item, version};
        }
    }
    console.log('target :', target);
    return target;
}

// 判断是否为移动端
function isMobile() {
	if(navigator.userAgent.match(/Android/i)
		|| navigator.userAgent.match(/webOS/i)
		|| navigator.userAgent.match(/iPhone/i)
		|| navigator.userAgent.match(/iPad/i)
		|| navigator.userAgent.match(/iPod/i)
		|| navigator.userAgent.match(/BlackBerry/i)
	    || navigator.userAgent.match(/Windows Phone/i)
    ) {
        return true;
    } else {
        return false;
    }
}

// 判断浏览器类型、版本是否符合
export function isSupport(chrome, firefox) {
    const {name, version} = getCurrentBrowserProfile();
    if (isMobile()) {
        return true;
    } else {
        if ((name === 'chrome' && version >= 72) || (name === 'firefox' && version >= 72)) {
            console.log('name, version :', name, version);
            return true;
        }
        return false;
    }
}

// 获取系统类型 mac || window
export function getSystemType() {
    const userAgent = navigator.userAgent;
    if (/macintosh|mac os x/i.test(userAgent)) {
        return 'mac';
    } else if (/windows|win32/i.test(userAgent)) {
        return 'windows';
    } else {
        return 'other';
    }
}

// 设置设备的输出源（摄像头、麦克风、扬声器）
export function setDeviceStatus(list) {
    list.forEach(item => {
        localStorage.setItem(item.text, `${item.label},${item.deviceId}`);
        if (item.text === 'videoDevice') {
            console.log('BRTC_ReplaceVideo,deviceId:' + item.deviceId);
            BRTC_ReplaceVideo(item.deviceId);
        } else if (item.text === 'audioInDevice') {
            console.log('BRTC_ReplaceAudio,deviceId:' + item.deviceId);
            BRTC_ReplaceAudio(item.deviceId);
        } else if (item.text === 'audioOutDevice') {
            console.log('BRTC_SetAudioOutputDevice,deviceId:' + item.deviceId);
            BRTC_SetAudioOutputDevice(item.deviceId);
        }
    });
}

// 更改时间格式
function changeTimeFormat(time) {
    let newTime = time.split(' ');
    let meetIngTime = '';
    if (newTime[0] === newTime[3]) { // 同一天
        meetIngTime = `${newTime[0]} ${newTime[1]} - ${newTime[4]}`
    } else { // 跨天
        meetIngTime = `${newTime[0]} ${newTime[1]} - 次日 ${newTime[4]}`
    }
    return meetIngTime;
}

// 获取cookie
export function getCookie(name) {
    const reg = new RegExp("(^| )"+name+"=([^;]*)(;|$)");
    const arr = document.cookie.match(reg);
    if (arr) {
        const data = unescape(arr[2]);
        return data;
    }
    return "";
}

// deepCopy: (params) => { // 深拷贝
//     return JSON.parse(JSON.stringify(params));
// },

// 查找数组中最大的值
export function findMaxValue(list) {
    let max = 0;
    if (list[0].ssrcs && list[0].ssrcs[0]) {
        max = Number(list[0].ssrcs[0].audioLevel);
    }
    let num = 0;
    list.forEach((item, ind) => {
        if (item.ssrcs && item.ssrcs[0]) {
            if (max < parseFloat(item.ssrcs[0].audioLevel)) {
                num = ind;
                max = parseFloat(item.ssrcs[0].audioLevel);
            } else {
                num = num;
                max = max;
            }
        }
    });
    return list[num]; // 返回最大声音的那一项
}

// 两个时间戳相减后获取天，时，分，秒
// //                                     开始时间戳, 当前时间戳
export function getDHMSFromTwoTimeStamps(faultDate, completeTime) {
    let stime = Date.parse(new Date(faultDate));
    let etime = Date.parse(new Date(completeTime));
    let usedTime = etime - stime;  //两个时间戳相差的毫秒数
    let days = Math.floor(usedTime/(24*3600*1000));
    //计算出小时数
    let leave1 = usedTime % (24 * 3600 * 1000);    //计算天数后剩余的毫秒数
    let hours = Math.floor(leave1 / (3600 * 1000));
    //计算相差分钟数
    let leave2 = leave1 % (3600 * 1000);        //计算小时数后剩余的毫秒数
    let minutes = Math.floor(leave2 / (60 * 1000));
    // 计算相差秒数
    let leave3 = leave2 % (60 * 1000);
    let seconds = Math.floor(leave3 / 1000);
    // let time = days + '天' + hours + '时' + minutes + '分' + seconds + '秒';
    let time = {d: days, h: hours, m: minutes, s: seconds};
    // 返回间隔的天，时，分，秒
    return time;
    // 页面中用法，先引入utils:import utils from '@/common/utils';
    // 然后直接调用该方法:utils.getDHMSFromTwoTimeStamps(1576377464000, 1576477464000);
}

// 数字正则
export function NumberReg() {
    return /^[0-9]\d*$/;
}

// 获取之前的一段时间:单位 (小时)
export function getBeforeTime(pix) {
    const start = new Date();
    return start.setTime(start.getTime() - 3600 * 1000 * pix); // 时间戳
}

// 格式化时间
export function formateDateTimeToSecond(time) {
    var date = new Date(time);
    let year = date.getFullYear();
    let month = date.getMonth() + 1;
    let day = date.getDate();
    let hours = date.getHours();
    let miu = date.getMinutes();
    let sec = date.getSeconds();
    month = month < 10 ? '0' + month : month;
    day = day < 10 ? '0' + day : day;
    hours = hours < 10 ? '0' + hours : hours;
    miu = miu < 10 ? '0' + miu : miu;
    sec = sec < 10 ? '0' + sec : sec;
    return  year + '/' + month + '/' + day + ' ' + hours + ':' + miu + ':' + sec;
}

// 获取整天的日期
export function getallday(time,defaultTime) {
    var date = new Date();
    let year = date.getFullYear();
    let month = date.getMonth() + 1;
    let day = date.getDate()-time;
    month = month < 10 ? '0' + month : month;
    day = day < 10 ? '0' + day : day;
    return  year + '-' + month + '-' + day + ' ' + defaultTime;
}

// 根据传入的北京时间转换成当前时区的时间,time：北京时间(2018-5-14 12:00:00) type：最后想要-还是/
function accoBJTransTimeZone(time, type) {
    let d = new Date();
    let utc = d.getTimezoneOffset() / 60;
    return formatTime(time, -utc, type);
}

// t传入的时间参数,utc传入的时区参数
function formatTime(t, utc, type) {
    if(!t)return
    let d = new Date();
    //获得本地时区
    utc = utc ? parseFloat(utc) : d.getTimezoneOffset() / 60;
    let time = new Date(t);
    //let time=new Date(t.toString().substring(0,4),t.toString().substring(5,7),t.toString().substring(8,10),t.toString().substring(11,13),t.toString().substring(14,16),t.toString().substring(17,19))
    //转换传入时间为本地时间（默认传入服务器时间为东八区时间）
    time.setHours(time.getHours() + (utc - 8));
    //输出时间
    let yy = time.getFullYear();
    let MM = time.getMonth()+1;
    // +1使用new Date(YYYY,MM,DD,hh,mm,ss)这种方式月数不会默认-1但是使用new Date('YYYY-MM-DD hh:mm:ss')会默认-1
    MM = MM < 10 ? '0' + MM : MM;
    let dd = time.getDate();
    dd = dd < 10 ? '0' + dd : dd;
    let hh = time.getHours();
    hh = hh < 10 ? '0' + hh : hh;
    let mm = time.getMinutes();
    mm = mm < 10 ? '0' + mm : mm;
    let ss = time.getSeconds();
    ss = ss < 10 ? '0' + ss : ss;
    let date = yy + type + MM + type + dd + ' ' + hh + ':' + mm + ':' + ss;
    // console.log(date);
    return date;
}

// 间隔多久后执行  单位是毫秒
export function sleep (numberMillis) {
    var now = new Date();
    var exitTime = now.getTime() + numberMillis;
    while (true) {
        now = new Date();
        if (now.getTime() > exitTime) {
            return;
        }
    }
}

// 会议时间按对应的时区展示
export function organizeCrossTime(time) {
    let splitTime = time.split(' - ');
    let start = splitTime[0] + ':00';
    let end = splitTime[1] + ':00';
    let zoneStart = accoBJTransTimeZone(start, '/');
    let zoneEnd = accoBJTransTimeZone(end, '/');
    let zoneStartTime = zoneStart.split(':')[0] + ':' + zoneStart.split(':')[1];
    let zoneEndTime = zoneEnd.split(':')[0] + ':' + zoneEnd.split(':')[1];
    let showTime = '';
    let newTime = (zoneStartTime + ' - ' + zoneEndTime).split(' ');
    if (newTime[0] === newTime[3]) { // 同一天
        showTime = `${newTime[0]} ${newTime[1]} - ${newTime[4]}`
    } else { // 跨天
        showTime = `${newTime[0]} ${newTime[1]} - ${newTime[3]} ${newTime[4]}`
    }
    return {showTime, zoneStart};
}

// 获取url的query参数
export function getRequestUrl() {
    let url = location.href.split('?')[1]; //获取url中"?"符后的字串
    let theRequest = {};
    if (!url) {
        return theRequest;
    }
    let strs = url.split("&");
    for(let i = 0; i < strs.length; i ++) {
        theRequest[strs[i].split("=")[0]] = decodeURIComponent(strs[i].split("=")[1]);
    }
    return theRequest;
}

// 调用sdk成功后重新设置麦克风和扬声器的deviceId
export function sdkSuccessSetDeviceId(audioId, audioOutId) {
    window.BRTC_DeviceTest_ReplaceAudio(audioId);
    window.BRTC_DeviceTest_SetAudioOutputDevice(audioOutId);
}

// 设置切换设备deviceId
export function setSdkReplaceDeviceId(type, id, flag) {
    if (type === 'video') { // 摄像头
        window.BRTC_ReplaceVideo(id);
        window.BRTC_DeviceTest_ReplaceVideo(id);
    } else if (type === 'audio') { // 麦克风
        window.BRTC_ReplaceAudio(id);
        window.BRTC_DeviceTest_ReplaceAudio(id);
        if (flag) { // 如果麦克风为静音状态下切换麦克风设备，此时麦克风静音就会失效，需要重新设置一下麦克风静音
            setTimeout(() => {
                window.BRTC_MuteMicphone(true);
            }, 100);
        }
    } else { // 扬声器
        window.BRTC_SetAudioOutputDevice(id);
        window.BRTC_DeviceTest_SetAudioOutputDevice(id);
    }
}

// 设置初始语言
export function setCreateLanguage(list) {
    // 读取local是否存在language字段
    const chooseLanguage = localStorage.getItem('language');
    let languageValue = {};
    if (chooseLanguage) { // 如果存在，查找和语言列表里能否对应
        const item = list.find(item => chooseLanguage === item.deviceId);
        languageValue = item ? item : list[0]; // 对应传找到的那一项，不对应传数组第一项
    } else { // local里不存在也传数组第一项
        languageValue = list[0];
    }
    // 把最后得到的值给组件赋值
    return languageValue;
}
