// eslint-disable-next-line no-unused-vars
const path = require('path');
// eslint-disable-next-line prettier/prettier
const debug = process.env.NODE_ENV !== 'production';
// const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
console.log('process.env.ENV:', process.env.ENV);
console.log('process.env.NODE_ENV:', process.env.NODE_ENV);
let outputDir = '';
let assetsPublicPath = '';
let {ENV, VUE_APP_TEST_URL} = process.env;
if (ENV === 'rd') {
    outputDir = 'rd';
    assetsPublicPath = './';
} else if (ENV === 'production') {
    outputDir = 'dist';
} else {
    outputDir = ENV;
}
module.exports = {
    /*
        // 详细：https://cli.vuejs.org/zh/config/#publicpath
        相对 publicPath 的限制
        相对路径的 publicPath 有一些使用上的限制。在以下情况下，应当避免使用相对 publicPath:
            当使用基于 HTML5 history.pushState 的路由时；
            当使用 pages 选项构建多页面应用时。
    */
    publicPath: assetsPublicPath, // 跟域上下文目录
    outputDir: outputDir, // 构建输出目录
    assetsDir: 'assets', // 静态资源目录（js, css, img, fonts）
    lintOnSave: true, // 是否开启eslint保存检测，有效值 true | false | 'error'
    runtimeCompiler: true, // 运行时版本是否需要编译
    transpileDependencies: [], // 默认babel-loader忽略node_modules, 这里可增加例外的包依赖包名
    productionSourceMap: false, // 是否在构建生产包时生成 sourceMap 文件，false将提高构建速度
    configureWebpack: config => {
        // webpack配置，值位对象时会合并配置，为方法时会改写配置
        // https://cli.vuejs.org/zh/guide/webpack.html#%E7%AE%80%E5%8D%95%E7%9A%84%E9%85%8D%E7%BD%AE%E6%96%B9%E5%BC%8F
        if (debug) {
            // 为开发环境修改配置...
            config.devtool = 'cheap-module-eval-source-map';
        } else {
            // config.optimization = {
            //     minimizer: [
            //         new UglifyJsPlugin({
            //             uglifyOptions: {
            //                 compress: {
            //                     // warnings: false,
            //                     // eslint-disable-next-line
            //                     drop_console: true, // console
            //                     // drop_debugger: false,
            //                     // eslint-disable-next-line
            //                     pure_funcs: ['console.log'] // 移除console
            //                 }
            //             }
            //         })
            //     ]
            // };
            // 为生产环境修改配置...
            // config.output.filename = 'assets/js/[name].[contenthash:8].min.js';
            // config.output.chunkFilename
            //     = 'assets/js/[name].[contenthash:8].min.js';
        }
        Object.assign(config, {
            resolve: {
                extensions: ['.js', '.vue', '.json'], // 省略文件后缀
                alias: {
                    // 配置引入的简易路径
                    '@': path.join(__dirname, 'src'),
                    'component': path.join(__dirname, 'src/views/component'),
                    'page': path.join(__dirname, 'src/views/page')
                }
            }
        });
    },
    // eslint-disable-next-line no-unused-vars
    chainWebpack: config => {
        // webpack链接API，用户生成和修改webpack配置，https://github.com/vuejs/vue-cli/blob/dev/docs/webpack.md

        // 生产环境配置
        config.plugin('html').tap(args => {
            // 不打包html文件
            args[0].minify = false;
            return args;
        });
        // 图片配置
        // test: /\.(png|jpe?g|gif|webp)(\?.*)?$/
        config.module
            .rule('images')
            .use('url-loader')
            .loader('url-loader')
            .tap(options => {
                // 1024 === 1kb
                // 4 * 1024 === 4096  4kb
                // limit默认4096 === 4kb
                // 40kb以内会被压缩成base64格式
                return {...options, limit: 1024 * 40};
            });
        // svg图配置
        // test: /\.(svg)(\?.*)?$/

        const svgRule = config.module.rule('svg');

        // 清除已有的所有 loader。
        // 如果你不这样做，接下来的 loader 会附加在该规则现有的 loader 之后。
        svgRule.uses.clear();
        config.module
            .rule('svg')
            .use('url-loader')
            .loader('url-loader')
            .tap(() => {
                // limit默认4096 === 1024 * 4  === 4kb
                // / 40kb以内会被压缩成base64格式
                return {
                    limit: 1024 * 40,
                    fallback: {
                        loader: 'file-loader',
                        options: {
                            name: 'assets/img/[name].[hash:8].[ext]'
                        }
                    }
                };
            });
    },
    parallel: require('os').cpus().length > 1,
    // pluginsOptions: {
    //   // 第三方插件配置
    // },
    pwa: {
        // 单页插件相关配置 https://github.com/vuejs/vue-cli/tree/dev/packages/%40vue/cli-plugin-pwa
    },
    css: {
        loaderOptions: {
            // eslint-disable-next-line max-len
            // https://cli.vuejs.org/zh/guide/css.html#%E5%90%91%E9%A2%84%E5%A4%84%E7%90%86%E5%99%A8-loader-%E4%BC%A0%E9%80%92%E9%80%89%E9%A1%B9
            // 给 sass-loader 传递选项
            // sass: {
            //     // @/ 是 src/ 的别名
            //     // 所以这里假设你有 `src/variables.scss` 这个文件
            //     prependData: '@import "@/styles/variables.scss";'
            // }
        }
    },
    devServer: {
        open: true,
        host: 'localhost',
        port: 8081,
        https: false,
        hotOnly: true,
        proxy: {
            // 配置跨域
            [VUE_APP_TEST_URL]: {
                target: VUE_APP_TEST_URL,
                ws: true,
                changeOrigin: true,
                pathRewrite: {
                    [`^${VUE_APP_TEST_URL}`]: ''
                }
            }
        }
    }
};
